import { LitElement, html, property, query } from 'lit-element';
import Prism from 'prismjs';
import 'prismjs/components/prism-bash';
import 'prismjs/components/prism-javascript';
import 'prismjs/components/prism-json';
import 'prismjs/components/prism-markup';
import 'prismjs/components/prism-css';
import 'prismjs/components/prism-yaml';

if (Prism) {
  /* eslint no-console: 0 */
  console.log('Prism loaded');
  window.Prism = Prism;
}

import style from './fc-highlight.css';

const prettifyXML = xml => {
  let formatted = '';
  const reg = /(>)(<)(\/*)/g;
  xml = xml.replace(/\s\s+/g, ' ')
    .replace(/&quot;/g, '"')
    .replace(' on-', ' @')
    .replace(/ {2}/g, ' ')
    .replace(/> </g, '><')
    .replace(reg, '$1\r\n$2$3')
    .trim();

  let pad = 0;
  xml.split('\r\n').map(function(node, index) {
    let indent = 0;
    if (node.match(/.+<\/\w[^>]*>$/)) {
      indent = 0;
    } else if (node.match(/^<\/\w/)) {
      if (pad !== 0) {
        pad -= 1;
      }
      /* eslint no-useless-escape: 0 */
    } else if (node.match(/^<\w[^>]*[^\/]>.*$/)) {
      indent = 1;
    } else {
      indent = 0;
    }

    let padding = '';
    /* eslint no-loops/no-loops: 0 */
    for (let i = 0; i < pad; i++) {
      padding += '  ';
    }

    formatted += `${padding + node}\r\n`;
    pad += indent;
  });

  return formatted.replace(/&quot;/g, '"');
};

/**
 * An element by Forter
 *
 * ## Usage
 *
 * ```html
 * <script>
 *   import '@forter/highlight';
 * </script>
 *
 * <fc-highlight>
 *   {
 *     "b": 1,
 *     "c": null,
 *     "d": "a",
 *     "e": true
 *   }
 * </fc-highlight>
 * ```
 *
 * ## Examples
 *
 * ```html
 * <!-- json -->
 * <fc-highlight>
 *   {"a": {"b": 1, "c": null, "d": "a", "e": false, "f": true, "h": 0 }}
 * </fc-highlight>
 *
 * <!-- html -->
 * <fc-highlight>
 *   <div class="hero">
 *     <div class="hero"></div>
 *     <div class="hero"></div>
 *    </div>
 * </fc-highlight>
 *
 * <!-- css -->
 * <fc-highlight>
 *   .super-class {
 *     font-size: 15px;
 *    }
 * </fc-highlight>
 *
 * <!-- javascript -->
 * <fc-highlight>
 *   import html from 'lit-html
 * </fc-highlight>
 *
 * <!-- yaml -->
 * <fc-highlight>
 *   includes:
 *    - goldi
 *    - ori
 * </fc-highlight>
 *
 * <!-- bash -->
 * <fc-highlight>
 *   export TEST="123"
 * </fc-highlight>
 * ```
 *
 * @element fc-highlight
 * @cssprop --fc-highlight-attr-color - attr color. example: `var(--yellow-9)`, default: `var(--orange-9)`
 * @cssprop --fc-highlight-tag-color - tag color. example: `var(--yellow-9)`, default: `var(--green-9)`
 * @cssprop --fc-highlight-null-color - null color. example: `var(--yellow-9)`, default: `var(--red-9)`
 * @cssprop --fc-highlight-value-color - value color. example: `var(--yellow-9)`, default: `var(--cyan-9)`
 * @slot - content to highlight. like `{"a": {"b": 1, "c": null, "d": "a", "e": false, "f": true, "h": 0 }}`
 *
 */
export class FcHighlight extends LitElement {
  /**
   * language of code language to highlight
   * @type {'html'|'bash'|'javascript'|'json'|'css'|'yaml'}
   * @attr
   */
  @property({ type: String, reflect: true }) language;

  @property({ type: String }) content;

  @query('#code') codeNode;

  @query('#code-inner') codeInnerNode;

  static is = 'fc-highlight';

  static styles = [style];

  codeHighlighter() {
    const { markup, type, prettify } = this;
    this.codeInnerNode.textContent = prettify(markup, type);
    window.Prism.highlightAllUnder(this.codeNode);
  }

  updated() {
    this.codeHighlighter();
  }

  prettify(innerHTML, language) {
    if (language === 'json') {
      return JSON.stringify(JSON.parse(innerHTML), 2, 2);
    }
    if (language === 'html') {
      return prettifyXML(innerHTML.trim());
    }
    return innerHTML;
  }

  getLanguage(innerHTML, language) {
    if (language) {
      return language;
    }
    if (innerHTML.startsWith('{') && innerHTML.endsWith('}')) {
      return 'json';
    }
    if (innerHTML.startsWith('<') && innerHTML.endsWith('>')) {
      return 'html';
    }
    if (!innerHTML.startsWith('{') && innerHTML.endsWith('}')) {
      return 'css';
    }
    if (innerHTML.startsWith('import ')) {
      return 'javascript';
    }
    if (innerHTML.split('\n')[0].endsWith(':')) {
      return 'yaml';
    }
    return 'bash';
  }

  getContent(textContent, content, innerHTML) {
    let markup = (content || (textContent || '').trim() || innerHTML);

    if (typeof markup !== 'string') {
      markup = JSON.stringify(markup);
    }
    return markup;
  }

  /** @inheritdoc */
  render() {
    const { language, textContent, innerHTML, getContent, getLanguage, content } = this;
    this.markup = getContent(textContent, content, innerHTML);
    this.type = getLanguage(this.markup, language);
    return html`
      <code hidden>
        <slot></slot>
      </code>
      <div id="code">
        <pre class=${`language-${this.type}`}>
          <code class="title">${this.type}</code>
          <code id="code-inner" class=${`language-${this.type}`}>
          </code>
        </pre>
      </div>
    `;
  }
}
